@@ -13,7 +13,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.03';
+$VERSION = '1.07';
 
 sub ProcessLeaf($$$);
 
@@ -21,11 +21,11 @@ sub ProcessLeaf($$$);
     PROCESS_PROC => \&ProcessLeaf,
     GROUPS => { 0 => 'Leaf', 2 => 'Camera' },
     NOTES => q{
-These tags are found in .MOS images from Leaf digital camera backs as
-written by Creo Leaf Capture.  They exist within the Leaf-specific directory
-structure of EXIF tag 0x8606. The tables below list observed Leaf tags,
-however ExifTool will extract any tags found in the Leaf directories even if
-they don't appear in these tables.
+        These tags are found in .MOS images from Leaf digital camera backs as
+        written by Creo Leaf Capture.  They exist within the Leaf-specific directory
+        structure of EXIF tag 0x8606.  The tables below list observed Leaf tags,
+        however ExifTool will extract any tags found in the Leaf directories even if
+        they don't appear in these tables.
     },
     icc_camera_profile => {
         Name => 'ICC_Profile',
@@ -47,6 +47,7 @@ they don't appear in these tables.
     },
     JPEG_preview_data => {
         %Image::ExifTool::previewImageTagInfo,
+        Groups => { 2 => 'Preview' },
     },
     JPEG_preview_info => 'PreviewInfo',
     icc_camera_to_tone_space_flow => {
@@ -73,7 +74,7 @@ they don't appear in these tables.
     back_serial_number => {
         Name => 'BackSerial',
         Description => 'Back Serial Number',
-        PrintConv => '$val =~ s/ .*//; $val',
+        PrintConv => '$val =~ s/ .*//s; $val',
     },
     image_offset => { Format => 'int16u' },
 );
@@ -110,7 +111,7 @@ they don't appear in these tables.
     CaptProf_serial_number  => {
         Name => 'CaptureSerial',
         Description => 'Capture Serial Number',
-        PrintConv => '$val =~ s/ .*//; $val',
+        PrintConv => '$val =~ s/ .*//s; $val',
     },
     CaptProf_image_offset       => {},
     CaptProf_luminance_consts   => {},
@@ -339,9 +340,10 @@ they don't appear in these tables.
     WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
     NOTES => q{
         Leaf writes a TIFF-format sub-IFD inside IFD0 of a MOS image.  No tags in
-        this sub-IFD are currently known, except for tag 0x8606 which really
-        shouldn't be here anyway (so it doesn't appear in the table below) because
-        it duplicates a reference to the same data of tag 0x8606 in IFD0.
+        this sub-IFD are currently known, except for tags 0x8602 and 0x8606 which
+        really shouldn't be here anyway (so they don't appear in the table below)
+        because they duplicate references to the same data from tags with the same
+        ID in IFD0.
     },
 );
 
@@ -394,17 +396,17 @@ they don't appear in these tables.
 # Returns: 1 on success, otherwise returns 0 and sets a Warning
 sub ProcessLeaf($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $dirStart = $$dirInfo{DirStart} || 0;
     my $dirLen = $$dirInfo{DirLen} || $$dirInfo{DataLen} - $dirStart;
     my $dirEnd = $dirStart + $dirLen;
-    my $verbose = $exifTool->Options('Verbose');
+    my $verbose = $et->Options('Verbose');
     my $pos = $dirStart;
     my $hdrLen = 52;    # header length for PKTS information
     my $success;
 
-    $verbose and $exifTool->VerboseDir('Leaf');
+    $verbose and $et->VerboseDir('Leaf');
     for (;;) {
         last if $pos + $hdrLen > $dirEnd;
         my $header = substr($$dataPt, $pos, $hdrLen);
@@ -413,13 +415,13 @@ sub ProcessLeaf($$$)
         my $size = Get32u(\$header, 48);
         $pos += $hdrLen;
         if ($pos + $size > $dirEnd) {
-            $exifTool->Warn('Truncated Leaf data');
+            $et->Warn('Truncated Leaf data');
             last;
         }
         my $tag = substr($header, 8, 40);
-        $tag =~ s/\0.*//;
+        $tag =~ s/\0.*//s;
         next unless $tag;
-        my $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $tag);
+        my $tagInfo = $et->GetTagInfo($tagTablePtr, $tag);
         # generate tag info for unknown tags
         my $val;
         if ($tagInfo and $$tagInfo{Format}) {
@@ -437,7 +439,7 @@ sub ProcessLeaf($$$)
                     SubDirectory => { TagTable => 'Image::ExifTool::Leaf::Unknown' },
                 };
             } elsif ($tagTablePtr ne \%Image::ExifTool::Leaf::Main or
-                     $exifTool->Options('Unknown'))
+                     $et->Options('Unknown'))
             {
                 $tagInfo = {
                     Name => $name,
@@ -447,10 +449,10 @@ sub ProcessLeaf($$$)
                 # make tags in main table unknown because they tend to be binary
                 $$tagInfo{Unknown} = 1 if $tagTablePtr eq \%Image::ExifTool::Leaf::Main;
             }
-            $tagInfo and Image::ExifTool::AddTagToTable($tagTablePtr, $tag, $tagInfo);
+            $tagInfo and AddTagToTable($tagTablePtr, $tag, $tagInfo);
         }
         if ($verbose) {
-            $exifTool->VerboseInfo($tag, $tagInfo,
+            $et->VerboseInfo($tag, $tagInfo,
                 Table   => $tagTablePtr,
                 Value   => $val,
                 DataPt  => $dataPt,
@@ -469,16 +471,16 @@ sub ProcessLeaf($$$)
                     DirName => 'Leaf PKTS',
                 );
                 my $subTable = GetTagTable($tagInfo->{SubDirectory}->{TagTable});
-                $exifTool->ProcessDirectory(\%subdirInfo, $subTable);
+                $et->ProcessDirectory(\%subdirInfo, $subTable);
             } else {
                 $val =~ tr/\n/ /;   # translate newlines to spaces
                 $val =~ s/\0+$//;   # remove null terminators
-                $exifTool->FoundTag($tagInfo, $val);
+                $et->FoundTag($tagInfo, $val);
             }
         }
         $pos += $size;
     }
-    $success or $exifTool->Warn('Bad format Leaf data');
+    $success or $et->Warn('Bad format Leaf data');
     return $success;
 }
 
@@ -502,7 +504,7 @@ Capture.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2017, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
